/*

#Basic script in python to plot your results
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
import matplotlib.patches as patches

time, Pars, Pluxr, Pci, Phyb, arsr, luxi, luxr, hs, p, ci, rfp, ars  = np.loadtxt('results1.dat', usecols=range(0,13),unpack=True, skiprows=0) # Reads the data from resultsi.dat file

fig = plt.figure()
ax1 = fig.add_subplot(1,1,1)

ax1.set_xlabel('Time (minutes)')
ax1.set_ylabel('Number of Molecules')
ax1.grid(color='lightgrey', linestyle='-.')

ax1.plot(time, rfp, label = 'rfp')
ax1.legend(numpoints=1)
plt.show()

*/

// Code based on Aberdeen 2009 code for IGEM competition

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>


#define dim 12 
#define N 80000 //Total time (minutes) = N*h/60
#define time_step 0.1

void RungeKutta(float y[],float h,float dy[]);
float *diff_eq(float y[], float dy[]);

int main()
{
  float y[dim],dy[dim];
  double t;
  float t_ars_input;
  int N_ars_input;


  
  // Defines multiple files for multiple outputs (different levels of Arsenic concentration)
  int numfiles=2;

  FILE *files[numfiles];
  for (int l = 0; l < numfiles; l++)
  {

    char filename[20];
    sprintf(filename, "results%d.dat", l); //sprintf changes the name of the filename
    files[l] = fopen(filename, "w");

    printf("Doing iteration for file results%d.dat: \n", l);
    
    // Initial Conditions

    for(int i=1; i<dim; i++)
    {
        y[i]=0.0;
        dy[i]=0.0;
    }    

    for(int j=1;j<N;j++)
    {
      if (j%1000 == 0)
      {
        printf("Iteration: %i \n", j);
      }
      //Define time (minutes)
      t=((float)j*time_step)/60.;
      y[0] = t;

      // Arsenic input (in this case at t=0)

      t_ars_input = 10; //Time in minutes when arsenic is detected
      N_ars_input = (t_ars_input/0.1)*60.;

      if (j>N_ars_input)
      {
        y[12]=1000*l; 
      }

      
      fprintf(files[l],"%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\n",y[0],y[1],y[2],y[3],y[4],y[5],y[6],y[7],y[8],y[9],y[10],y[11],y[12]);     

      // Integrating
               
      RungeKutta(y,time_step,dy);

    }
    fclose(files[l]);
  }
  return 0;
}

float *diff_eq(float y[], float dy[])
{

/*
  Variables order
  y[0] = time (minutes)
  y[1] = Pars
  y[2] = Pluxr
  y[3] = Pci
  y[4] = Phyb
  y[5] = arsr
  y[6] = luxi
  y[7] = luxr
  y[8] = hs
  y[9] = p
  y[10] = ci
  y[11] = rfp
  y[12] = ars
*/

/*
  PARAMETERS
*/

/* This coefficients are not available in literature so we estimated them based in similar genetic components */

double beta_Pars                 = 0.4;   // promotor Pars
double beta_Pluxr                = 0.018; // promotor constituivo PluxR
double beta_Pci                  = 0.5;   // promotor Pci
double beta_Prfp                 = 0.44;  // promotor hybrido

double K_ars                     = 1200; 
double K_arsr                    = 700;  
double K_hybi                    = 300;  
double K_hyba                    = 700;  

double n_ars                     = 1; //no se si tiene cooperatividad pero la asumo por ahora; al final no
double n_arsr                    = 1; //no se si tiene cooperatividad pero la asumo por ahora; al final no


double alpha_hsl                 = 0.00017;
double alpha_p                   = 0.0002;
double alpha_arsr                = 0.0001; 

double dilution_rate             = (log(2)/1200); // Aberdeen team considered 30 min per cel division. We considered 20 min thus 1200 sec

/* Parameters taken from Aberdeen 2009 Model */

double mu                        = 0.45;
double theta_HSL                 = 0.24;
double rho                       = 10;

double K_ci                      = 7000; //guessed

double n_ci                      = 2;
double n_P                       = 2; 

double gamma_protein             = 0.1;

double k_on_P                    = 0.00001; 
double k_off_P                   = 0.00333;

double alpha_mrna                = 0.006;
double alpha_ci                  = 0.002888; // taged
double alpha_luxi                = 0.00288811; // taged
double alpha_luxr                = 0.0002;

      
// mRNA production equations      
	//Pars repressed by arsr y[4] and induced by ars y[11]
	dy[1] = 	rho*beta_Pars*(1/(1 + pow(  (y[5]/(K_arsr*(1+pow(y[12]/K_ars,n_ars)))),n_arsr ) )) - (alpha_mrna+dilution_rate)*y[1];
	dy[2] = 	rho*beta_Pluxr - (alpha_mrna+dilution_rate)*y[2];
	dy[3] =      	rho*beta_Pci*(		(pow(y[9],n_P)) / (pow(y[9],n_P)+pow(K_ci,n_P))	) - (alpha_mrna+dilution_rate)*y[3]; 
	dy[4] =      	rho*beta_Prfp*(		(pow(y[9],n_P))	/ (pow(y[9],n_P)+pow(K_hyba,n_P)) )*(1/(1+pow(y[10]/K_hybi,n_ci))) - (alpha_mrna+dilution_rate)*y[4];
      
// Protein production equations

	dy[5] =  gamma_protein*y[1] - (alpha_arsr+dilution_rate)*y[5];          
	dy[6] =  gamma_protein*y[1] - (alpha_luxi+dilution_rate)*y[6];
	dy[7] =  gamma_protein*y[2] + k_off_P*y[9] - k_on_P*(y[7]*y[8])  - (alpha_luxr+dilution_rate)*y[7];
	dy[8] =  mu*y[6] + (k_off_P+alpha_luxr)*y[9]- k_on_P*y[7]*y[8] + theta_HSL*(-y[8])  - (alpha_hsl+dilution_rate)*y[8]; 
	dy[9] =  k_on_P*y[8]*y[7] - k_off_P*y[9] - (alpha_p+dilution_rate)*y[9];  
	dy[10] = gamma_protein*y[3] - (alpha_ci+dilution_rate)*y[10];       
	dy[11] = gamma_protein*y[4] - (0+dilution_rate)*y[11];

  return dy;
}

void RungeKutta(float y[],float h,float dy[])
{
  float k1[dim],k2[dim],k3[dim],k4[dim],y1[dim],y2[dim],y3[dim],*p_dy;
  
  
  p_dy=diff_eq(y,dy);
  for(int i=1;i<dim;i++)
    {
      k1[i]=*(p_dy+i)*h;
      y1[i]=y[i]+0.5*k1[i];
    }

  p_dy=diff_eq(y1,dy);
  for(int i=1;i<dim;i++)
    {
      k2[i]=*(p_dy+i)*h;
      y2[i]=y[i]+0.5*k2[i];
    }
  p_dy=diff_eq(y2,dy);
  for(int i=1;i<dim;i++)
    {
      k3[i]=*(p_dy+i)*h;
      y3[i]=y[i]+k3[i];
    }
  p_dy=diff_eq(y3,dy);
  for(int i=1;i<dim;i++)
    {
      k4[i]=*(p_dy+i)*h;
    }
  
  for(int i=1;i<dim;i++)
    
    y[i]=y[i]+(k1[i]+2.*k2[i]+2.*k3[i]+k4[i])/6.;

 }

